//
//  Course.swift
//  UnilProject
//
//  Created by Arielle Moro on 17.12.16.
//  Copyright © 2016 Arielle. All rights reserved.
//

import Foundation

class Course : NSObject, CourseProtocol {

    var name: String
    var creditNumber: Int
    var professor: Professor
    var examPerformed: Bool
    var registrations: [Registration]
    
    //Constructor redefinition
    init(withName name: String, andCreditNumber creditNumber: Int, andProfessor professor: Professor) {
        self.name = name
        self.creditNumber = creditNumber
        self.professor = professor
        self.examPerformed = false
        self.registrations = []
    }
    
    //Funtion adding a new registration to the registration array of the course
    func addNewRegistration(registration: Registration) {
        registrations.append(registration)
    }
    
    //Funtion that indicates that an exam has been given for this course
    func performExam(){
        examPerformed = true
    }
    
    //Function that displays the stats of the exam
    func displayExamStats(){
        if(examPerformed) {
            var sumGrade: Double = 0.0
            var successNB: Int = 0
            var failureNB: Int = 0
            for registration in registrations {
                sumGrade += registration.grade
                if (registration.grade >= 4.0) {
                    successNB += 1
                } else {
                    failureNB += 1
                }
            }
            let successRate = Double(successNB) / Double(registrations.count)
            let failureRate = Double(failureNB) / Double(registrations.count)
            let examMean: Double = Double(sumGrade) / Double(registrations.count)
            if (examPerformed) {
                print("Exam stats => success rate: \(successRate) failure rate: \(failureRate) exam mean: \(examMean)")
            } else {
                print("The exam has not been performed yet!")
            }
        }
    }
    
    //Function returning all the registrations corresponding to the courseName
    func getRegistrations(courseName: String) -> [Registration] {
        return registrations.filter(){$0.course.name == courseName}
    }
    
    //Function setting a grade for a student that was registered for the course
    func setGrade(studentID: Int, grade: Double){
        if examPerformed {
            outer: for registration in registrations {
                let currentStudent = registration.student
                if (currentStudent.studentID == studentID) {
                    //Set the grade and if the course is validated for the student
                    registration.setGrade(grade: grade)
                    if (grade >= 4.0) {
                        registration.setValidation()
                    } else {
                        currentStudent.displayNotification(notification: "You have failed the course \(name) with the grade \(grade)/6.0, you must retake it next year.")
                    }
                    break outer
                }
            }
        }
    }
    
    //Function that ends the semester and reinit some variables
    func finishSemester(){
        if(examPerformed) {
            //Reinit the status of the exam
            examPerformed = false
            //Clean the registration array
            registrations = registrations.filter(){$0.grade < 4.0}
        }
    }
    
    //Description redefinition
    override var description : String {
        return "Course: \(name) given by " + professor.description + " - number of credits: \(creditNumber)"
    }
    
}
