/**
  * Created by ariellemoro on 12.12.16.
  */

import scala.collection.mutable.{ListBuffer, Map}

class Course(private var name: String, private var creditNumber: Int, private var professor: Professor) extends CourseInterface{

  private var examPerformed: Boolean = false
  private var registrations:ListBuffer[Registration] = new ListBuffer[Registration]

  //Getter: creditNumber
  def getCreditNumber(): Int = {
    return creditNumber
  }

  //Getter: name
  def getName(): String = {
    return name
  }

  //Function that indicates that an exam has been given for this course
  def performExam(): Unit = {
    examPerformed = true
  }

  //Function adding a new registration to the registration array of the course
  def addNewRegistration(registration: Registration): Unit = {
    registrations += registration
  }

  //Function that displays the stats of the exam
  def displayExamStats(): Unit = {
    if(examPerformed) {
      var sumGrade: Double = 0.0
      var successNB: Int = 0
      var failureNB: Int = 0
      for (registration <- registrations) {
        sumGrade += registration.getGrade()
        if (registration.getGrade() >= 4.0) {
          successNB += 1
        } else {
          failureNB += 1
        }
      }
      val successRate = successNB.toDouble / registrations.length.toDouble
      val failureRate = failureNB.toDouble / registrations.length.toDouble
      val examMean: Double = sumGrade / registrations.length.toDouble
      if (examPerformed) {
        println(s"Exam stats => success rate: $successRate failure rate: $failureRate exam mean: $examMean")
      } else {
        println(s"The exam has not been performed yet!")
      }
    }
  }

  //Function returning all the registrations corresponding to the courseName
  def getRegistrations(courseName:String): ListBuffer[Registration] = {
    return registrations.filter(_.getCourse().getName() == courseName)
  }

  //Function setting a grade for a student that was registered for the course
  def setGrade(studentID:Int, grade:Double): Unit = {
    if(examPerformed) {
      for (registration <- registrations) {
        var currentStudent: Student = registration.getStudent()
        if (currentStudent.getStudentID == studentID) {
          //Set the grade and if the course is validated for the student
          registration.setGrade(grade)
          if (grade >= 4.0) {
            registration.setValidation()
          } else {
            currentStudent.displayNotification(s"You have failed the course $name with the grade $grade/6.0, you must retake it next year.")
          }
        }
      }
    }
  }

  //Function that ends the semester and reinit some variables
  def finishSemester(): Unit = {
    if(examPerformed) {
      //Reinit the status of the exam
      examPerformed = false
      //Clean the registration array
      registrations = registrations.filter(_.getGrade() < 4.0)
    }
  }

  override def toString: String = {
    return s"Course: $name given by ${professor.toString} - number of credit: $creditNumber"
  }

}
